@@ -34,6 +34,7 @@ import javax.servlet.http.HttpSessionBindingListener;
 import javax.servlet.http.HttpSessionContext;
 import javax.servlet.http.HttpSessionEvent;
 
+import org.apache.felix.http.base.internal.HttpConfig;
 import org.apache.felix.http.base.internal.context.ExtServletContext;
 
 /**
@@ -87,21 +88,27 @@ public class HttpSessionWrapper implements HttpSession
      */
     private final boolean isNew;
 
-    public static boolean hasSession(final long contextId, final HttpSession session)
+    /**
+     * Session handling configuration
+     */
+    private final HttpConfig config;
+
+
+
+    public static boolean hasSession(final String contextName, final HttpSession session)
     {
-        final String sessionId = String.valueOf(contextId);
-        return session.getAttribute(ATTR_CREATED + sessionId) != null;
+        return session.getAttribute(ATTR_CREATED.concat(contextName)) != null;
     }
 
-    public static Set<Long> getExpiredSessionContextIds(final HttpSession session)
+    public static Set<String> getExpiredSessionContextNames(final HttpSession session)
     {
         final long now = System.currentTimeMillis();
 
-        final Set<Long> ids = new HashSet<Long>();
-        final Enumeration<String> names = session.getAttributeNames();
-        while (names.hasMoreElements())
+        final Set<String> names = new HashSet<>();
+        final Enumeration<String> attrNames = session.getAttributeNames();
+        while (attrNames.hasMoreElements())
         {
-            final String name = names.nextElement();
+            final String name = attrNames.nextElement();
             if (name.startsWith(ATTR_LAST_ACCESSED))
             {
                 final String id = name.substring(ATTR_LAST_ACCESSED.length());
@@ -111,77 +118,74 @@ public class HttpSessionWrapper implements HttpSession
 
                 if ((maxTimeout > 0) && (lastAccess + maxTimeout) < now)
                 {
-                    ids.add(Long.valueOf(id));
+                    names.add(id);
                 }
             }
         }
-        return ids;
+        return names;
     }
 
-    public static Set<Long> getSessionContextIds(final HttpSession session)
+    /**
+     * Get the names of all contexts using a session.
+     * @param session The underlying session
+     * @return The set of names
+     */
+    public static Set<String> getSessionContextNames(final HttpSession session)
     {
-        final Set<Long> ids = new HashSet<Long>();
-        final Enumeration<String> names = session.getAttributeNames();
-        while (names.hasMoreElements())
+        final Set<String> names = new HashSet<>();
+        final Enumeration<String> attrNames = session.getAttributeNames();
+        while (attrNames.hasMoreElements())
         {
-            final String name = names.nextElement();
+            final String name = attrNames.nextElement();
             if (name.startsWith(ATTR_LAST_ACCESSED))
             {
                 final String id = name.substring(ATTR_LAST_ACCESSED.length());
-                ids.add(Long.valueOf(id));
+                names.add(id);
             }
         }
-        return ids;
+
+        return names;
     }
 
     /**
      * Creates a new {@link HttpSessionWrapper} instance.
      */
-    public HttpSessionWrapper(final long contextId,
-            final HttpSession session,
+    public HttpSessionWrapper(final HttpSession session,
             final ExtServletContext context,
+            final HttpConfig config,
             final boolean terminate)
     {
+        this.config = config;
         this.delegate = session;
         this.context = context;
-        this.sessionId = String.valueOf(contextId);
-        this.keyPrefix = ATTR_PREFIX + this.sessionId + ".";
+        this.sessionId = context.getServletContextName();
+        this.keyPrefix = ATTR_PREFIX.concat(this.sessionId).concat(".");
 
-        if ( this.keyPrefix != null )
+        final String createdAttrName = ATTR_CREATED.concat(this.sessionId);
+
+        final long now = System.currentTimeMillis();
+        if ( session.getAttribute(createdAttrName) == null )
         {
-            final long now = System.currentTimeMillis();
-            if ( session.getAttribute(ATTR_CREATED + this.sessionId) == null )
-            {
-                this.created = now;
-                this.maxTimeout = session.getMaxInactiveInterval();
-                this.isNew = true;
+            this.created = now;
+            this.maxTimeout = session.getMaxInactiveInterval();
+            this.isNew = true;
 
-                session.setAttribute(ATTR_CREATED + this.sessionId, this.created);
-                session.setAttribute(ATTR_MAX_INACTIVE + this.sessionId, this.maxTimeout);
+            session.setAttribute(createdAttrName, this.created);
+            session.setAttribute(ATTR_MAX_INACTIVE.concat(this.sessionId), this.maxTimeout);
 
-                if ( context.getHttpSessionListener() != null )
-                {
-                    context.getHttpSessionListener().sessionCreated(new HttpSessionEvent(this));
-                }
-            }
-            else
-            {
-                this.created = (Long)session.getAttribute(ATTR_CREATED + this.sessionId);
-                this.maxTimeout = (Integer)session.getAttribute(ATTR_MAX_INACTIVE + this.sessionId);
-                this.isNew = false;
-            }
-
-            this.lastAccessed = now;
-            if ( !terminate )
-            {
-                session.setAttribute(ATTR_LAST_ACCESSED + this.sessionId, this.lastAccessed);
-            }
+            context.getHttpSessionListener().sessionCreated(new HttpSessionEvent(this));
         }
         else
         {
-            this.isNew = session.isNew();
-            this.lastAccessed = session.getLastAccessedTime();
-            this.created = session.getCreationTime();
+            this.created = (Long)session.getAttribute(createdAttrName);
+            this.maxTimeout = (Integer)session.getAttribute(ATTR_MAX_INACTIVE.concat(this.sessionId));
+            this.isNew = false;
+        }
+
+        this.lastAccessed = now;
+        if ( !terminate )
+        {
+            session.setAttribute(ATTR_LAST_ACCESSED.concat(this.sessionId), this.lastAccessed);
         }
     }
 
@@ -190,7 +194,7 @@ public class HttpSessionWrapper implements HttpSession
      */
     private String getKey(final String name)
     {
-        return this.keyPrefix == null ? name : this.keyPrefix.concat(name);
+        return this.keyPrefix.concat(name);
     }
 
     /**
@@ -231,10 +235,6 @@ public class HttpSessionWrapper implements HttpSession
                 while ( e.hasMoreElements() )
                 {
                     final String name = e.nextElement();
-                    if ( keyPrefix == null && !name.startsWith(PREFIX) )
-                    {
-                        return name;
-                    }
                     if ( name.startsWith(keyPrefix))
                     {
                         return name.substring(keyPrefix.length());
@@ -272,8 +272,12 @@ public class HttpSessionWrapper implements HttpSession
     @Override
     public String getId()
     {
-        this.checkInvalid();
-        return this.delegate.getId() + "-" + this.sessionId;
+        // no validity check conforming to the javadocs
+        if ( this.config.isUniqueSessionId() )
+        {
+            return this.delegate.getId().concat("-").concat(this.sessionId);
+        }
+        return this.delegate.getId();
     }
 
     @Override
@@ -300,13 +304,15 @@ public class HttpSessionWrapper implements HttpSession
     @Override
     public Object getValue(String name)
     {
+        this.checkInvalid();
         return this.getAttribute(name);
     }
 
     @Override
     public String[] getValueNames()
     {
-        final List<String> names = new ArrayList<String>();
+        this.checkInvalid();
+        final List<String> names = new ArrayList<>();
         final Enumeration<String> e = this.getAttributeNames();
         while ( e.hasMoreElements() )
         {
@@ -321,37 +327,52 @@ public class HttpSessionWrapper implements HttpSession
         this.checkInvalid();
 
         // session listener must be called before the session is invalidated
-        if (context.getHttpSessionListener() != null) {
-            context.getHttpSessionListener().sessionDestroyed(new HttpSessionEvent(this));
-        }
+        context.getHttpSessionListener().sessionDestroyed(new HttpSessionEvent(this));
 
-        if ( this.keyPrefix != null )
+        this.delegate.removeAttribute(ATTR_CREATED + this.sessionId);
+        this.delegate.removeAttribute(ATTR_LAST_ACCESSED + this.sessionId);
+        this.delegate.removeAttribute(ATTR_MAX_INACTIVE + this.sessionId);
+
+        // remove all attributes belonging to this session
+        final Enumeration<String> names = this.delegate.getAttributeNames();
+        while ( names.hasMoreElements() )
         {
-            this.delegate.removeAttribute(ATTR_CREATED + this.sessionId);
-            this.delegate.removeAttribute(ATTR_LAST_ACCESSED + this.sessionId);
-            this.delegate.removeAttribute(ATTR_MAX_INACTIVE + this.sessionId);
+            final String name = names.nextElement();
 
-            final Enumeration<String> names = this.delegate.getAttributeNames();
-            while ( names.hasMoreElements() )
-            {
-                final String name = names.nextElement();
-
-                if ( name.startsWith(this.keyPrefix) ) {
-                    this.removeAttribute(name.substring(this.keyPrefix.length()));
-                }
+            if ( name.startsWith(this.keyPrefix) ) {
+                this.removeAttribute(name.substring(this.keyPrefix.length()));
             }
         }
 
-        // if the session is empty we can invalidate
-        final Enumeration<String> names = this.delegate.getAttributeNames();
-        if ( !names.hasMoreElements() )
+        if ( this.config.isInvalidateContainerSession() )
         {
-            this.delegate.invalidate();
+            // if the session is empty we can invalidate
+            final Enumeration<String> remainingNames = this.delegate.getAttributeNames();
+            if ( (!remainingNames.hasMoreElements()) || (isRemainingAttributeAddedByContainer(remainingNames)))
+            {
+                this.delegate.invalidate();
+            }
         }
-
         this.isInvalid = true;
     }
 
+    private boolean isRemainingAttributeAddedByContainer(Enumeration<String> names){
+        final Set<String> attributeAddedByContainerSet = this.config.getContainerAddedAttribueSet() ;
+
+        if(attributeAddedByContainerSet != null && !attributeAddedByContainerSet.isEmpty()) {
+
+            while (names.hasMoreElements()) {
+
+                final String name = names.nextElement();
+                if (name == null || !attributeAddedByContainerSet.contains(name)) {
+                    return false;
+                }
+            }
+            return true ;
+        }
+        return false ;
+    }
+
     @Override
     public boolean isNew()
     {
@@ -362,6 +383,7 @@ public class HttpSessionWrapper implements HttpSession
     @Override
     public void putValue(final String name, final Object value)
     {
+        this.checkInvalid();
         this.setAttribute(name, value);
     }
 
@@ -373,7 +395,7 @@ public class HttpSessionWrapper implements HttpSession
         if ( oldValue != null )
         {
             this.delegate.removeAttribute(this.getKey(name));
-            if ( this.keyPrefix != null && oldValue instanceof HttpSessionBindingListener )
+            if ( oldValue instanceof HttpSessionBindingListener )
             {
                 ((HttpSessionBindingListener)oldValue).valueUnbound(new HttpSessionBindingEvent(this, name));
             }
@@ -387,6 +409,7 @@ public class HttpSessionWrapper implements HttpSession
     @Override
     public void removeValue(final String name)
     {
+        this.checkInvalid();
         this.removeAttribute(name);
     }
 
@@ -402,7 +425,7 @@ public class HttpSessionWrapper implements HttpSession
 
         final Object oldValue = this.getAttribute(name);
         // wrap http session binding listener to avoid container calling it!
-        if ( this.keyPrefix != null && value instanceof HttpSessionBindingListener )
+        if ( value instanceof HttpSessionBindingListener )
         {
             this.delegate.setAttribute(this.getKey(name),
                     new SessionBindingValueListenerWrapper((HttpSessionBindingListener)value));
@@ -411,7 +434,7 @@ public class HttpSessionWrapper implements HttpSession
         {
             this.delegate.setAttribute(this.getKey(name), value);
         }
-        if ( this.keyPrefix != null && value instanceof HttpSessionBindingListener )
+        if ( value instanceof HttpSessionBindingListener )
         {
             ((HttpSessionBindingListener)value).valueBound(new HttpSessionBindingEvent(this, name));
         }
@@ -432,14 +455,16 @@ public class HttpSessionWrapper implements HttpSession
     @Override
     public void setMaxInactiveInterval(final int interval)
     {
+        // no validity check conforming to the javadocs
         if ( this.delegate.getMaxInactiveInterval() < interval )
         {
             this.delegate.setMaxInactiveInterval(interval);
         }
-        if ( this.keyPrefix != null )
-        {
-            this.maxTimeout = interval;
+        this.maxTimeout = interval;
+        try {
             this.delegate.setAttribute(ATTR_MAX_INACTIVE + this.sessionId, interval);
+        } catch ( final IllegalStateException iae) {
+            // this might throw if delegate is invalid
         }
     }
 
@@ -467,4 +492,25 @@ public class HttpSessionWrapper implements HttpSession
             return listener;
         }
     }
+
+    @Override
+    public int hashCode()
+    {
+        return this.getId().concat(this.sessionId).hashCode();
+    }
+
+    @Override
+    public boolean equals(final Object obj)
+    {
+        if (this == obj)
+        {
+            return true;
+        }
+        if (obj == null || this.getClass() != obj.getClass() )
+        {
+            return false;
+        }
+        final HttpSessionWrapper other = (HttpSessionWrapper) obj;
+        return other.getId().concat(other.sessionId).equals(this.getId().concat(this.sessionId));
+    }
 }
