@@ -18,11 +18,10 @@ package org.apache.felix.http.base.internal.registry;
 
 import java.util.regex.Pattern;
 
-import javax.annotation.CheckForNull;
-import javax.annotation.Nonnull;
-
 import org.apache.felix.http.base.internal.handler.ServletHandler;
 import org.apache.felix.http.base.internal.service.HttpServiceFactory;
+import org.jetbrains.annotations.NotNull;
+import org.jetbrains.annotations.Nullable;
 
 /**
  * The path resolver factory creates a path resolver for a pattern.
@@ -38,7 +37,7 @@ import org.apache.felix.http.base.internal.service.HttpServiceFactory;
  */
 public abstract class PathResolverFactory {
 
-    public static @Nonnull PathResolver createPatternMatcher(@CheckForNull final ServletHandler handler, @Nonnull final String pattern)
+    public static @NotNull PathResolver createPatternMatcher(@Nullable final ServletHandler handler, @NotNull final String pattern)
     {
         if ( pattern.length() == 0 )
         {
@@ -63,7 +62,7 @@ public abstract class PathResolverFactory {
         return new ExactMatcher(handler, pattern);
     }
 
-    public static @Nonnull PathResolver createRegexMatcher(@Nonnull final String regex)
+    public static @NotNull PathResolver createRegexMatcher(@NotNull final String regex)
     {
         return new RegexMatcher(regex);
     }
@@ -156,6 +155,10 @@ public abstract class PathResolverFactory {
                 pr.requestURI = uri;
                 pr.handler = this.getServletHandler();
 
+                pr.matchedPattern = this.getPattern();
+                pr.mappingType = MappingType.CONTEXT_ROOT;
+                pr.matchValue = this.getPattern();
+
                 return pr;
             }
             return null;
@@ -177,6 +180,10 @@ public abstract class PathResolverFactory {
             pr.requestURI = uri;
             pr.handler = this.getServletHandler();
 
+            pr.matchedPattern = this.getPattern();
+            pr.mappingType = MappingType.DEFAULT;
+            pr.matchValue = "";
+
             return pr;
         }
     }
@@ -204,6 +211,11 @@ public abstract class PathResolverFactory {
                 pr.requestURI = uri;
                 pr.handler = this.getServletHandler();
 
+                pr.matchedPattern = this.getPattern();
+                pr.mappingType = MappingType.EXACT;
+                final int pos = uri.lastIndexOf("/");
+                pr.matchValue = uri.substring(pos + 1, uri.length());
+
                 return pr;
             }
             else if ( uri.startsWith(prefix) )
@@ -214,6 +226,11 @@ public abstract class PathResolverFactory {
                 pr.requestURI = uri;
                 pr.handler = this.getServletHandler();
 
+                pr.matchedPattern = this.getPattern();
+                pr.mappingType = MappingType.PATH;
+                final int pos = uri.lastIndexOf("/");
+                pr.matchValue = uri.substring(pos + 1, uri.length());
+
                 return pr;
             }
             return null;
@@ -246,6 +263,11 @@ public abstract class PathResolverFactory {
                 pr.requestURI = uri;
                 pr.handler = this.getServletHandler();
 
+                pr.matchedPattern = this.getPattern();
+                pr.mappingType = MappingType.EXACT;
+                final int pos = uri.lastIndexOf("/");
+                pr.matchValue = uri.substring(pos + 1, uri.length());
+
                 return pr;
             }
             return null;
@@ -281,6 +303,10 @@ public abstract class PathResolverFactory {
                 pr.requestURI = uri;
                 pr.handler = this.getServletHandler();
 
+                pr.matchedPattern = this.getPattern();
+                pr.mappingType = MappingType.PATH;
+                pr.matchValue = uri.length() > 0 ? uri.substring(1) : "";
+
                 return pr;
             }
             if ( uri.startsWith(this.prefix) )
@@ -291,6 +317,10 @@ public abstract class PathResolverFactory {
                 pr.requestURI = uri;
                 pr.handler = this.getServletHandler();
 
+                pr.matchedPattern = this.getPattern();
+                pr.mappingType = MappingType.PATH;
+                pr.matchValue = uri.substring(this.prefix.length(), uri.length());
+
                 return pr;
             }
             return null;
@@ -323,6 +353,11 @@ public abstract class PathResolverFactory {
                 pr.requestURI = uri;
                 pr.handler = this.getServletHandler();
 
+                pr.matchedPattern = this.getPattern();
+                pr.mappingType = MappingType.EXTENSION;
+                final int pos = uri.lastIndexOf("/");
+                pr.matchValue = uri.substring(pos + 1, uri.length() - this.extension.length());
+
                 return pr;
             }
             return null;
@@ -346,7 +381,7 @@ public abstract class PathResolverFactory {
         }
 
         @Override
-        public @CheckForNull PathResolution resolve(@Nonnull final String uri) {
+        public @Nullable PathResolution resolve(@NotNull final String uri) {
             if ( pattern.matcher(uri).matches() )
             {
                 final PathResolution pr = new PathResolution();
@@ -354,6 +389,11 @@ public abstract class PathResolverFactory {
                 pr.servletPath = uri;
                 pr.requestURI = uri;
 
+                // regex is not supported by servlet spec, we use PATH matching
+                pr.matchedPattern = this.getPattern();
+                pr.mappingType = MappingType.PATH;
+                pr.matchValue = uri;
+
                 return pr;
             }
             return null;
