@@ -42,12 +42,12 @@ import javax.servlet.ServletRegistration.Dynamic;
 import javax.servlet.SessionCookieConfig;
 import javax.servlet.SessionTrackingMode;
 import javax.servlet.descriptor.JspConfigDescriptor;
-import javax.servlet.http.MappingMatch;
 
 import org.apache.felix.http.base.internal.dispatch.RequestDispatcherImpl;
 import org.apache.felix.http.base.internal.dispatch.RequestInfo;
 import org.apache.felix.http.base.internal.handler.ServletHandler;
 import org.apache.felix.http.base.internal.logger.SystemLogger;
+import org.apache.felix.http.base.internal.registry.MappingType;
 import org.apache.felix.http.base.internal.registry.PathResolution;
 import org.apache.felix.http.base.internal.registry.PerContextHandlerRegistry;
 import org.apache.felix.http.base.internal.registry.ServletResolution;
@@ -321,7 +321,7 @@ public class SharedServletContextImpl implements ServletContext
         	resolution.handler = servletHandler;
             resolution.handlerRegistry = this.registry;
             // TODO - what is the path of a named servlet?
-            final RequestInfo requestInfo = new RequestInfo("", null, null, null, name, "", "", MappingMatch.EXACT, true);
+            final RequestInfo requestInfo = new RequestInfo("", null, null, null, name, "", "", MappingType.EXACT, true);
             dispatcher = new RequestDispatcherImpl(resolution, requestInfo);
         }
         else
@@ -359,7 +359,7 @@ public class SharedServletContextImpl implements ServletContext
             final RequestInfo requestInfo = new RequestInfo(pathResolution.servletPath, pathResolution.pathInfo, query,
                     UriUtils.concat(this.contextPath, encodedRequestURI),
                     pathResolution.handler.getName(), pathResolution.matchedPattern,
-                    pathResolution.matchValue, pathResolution.match,
+                    pathResolution.matchValue, pathResolution.mappingType,
                     false);
             dispatcher = new RequestDispatcherImpl(pathResolution, requestInfo);
         }
